@@ -1,11 +1,12 @@
 /*************************************************
 * Buffered EntropySource Source File             *
-* (C) 1999-2005 The Botan Project                *
+* (C) 1999-2007 The Botan Project                *
 *************************************************/
 
 #include <botan/buf_es.h>
 #include <botan/bit_ops.h>
 #include <botan/util.h>
+#include <algorithm>
 
 namespace Botan {
 
@@ -51,7 +52,7 @@ void Buffered_EntropySource::do_fast_poll()
 *************************************************/
 void Buffered_EntropySource::add_bytes(const void* entropy_ptr, u32bit length)
    {
-   const byte* bytes = (const byte*)entropy_ptr;
+   const byte* bytes = static_cast<const byte*>(entropy_ptr);
    while(length)
       {
       u32bit copied = std::min(length, buffer.size() - write_pos);
@@ -67,7 +68,7 @@ void Buffered_EntropySource::add_bytes(const void* entropy_ptr, u32bit length)
 *************************************************/
 void Buffered_EntropySource::add_bytes(u64bit entropy)
    {
-   add_bytes((const void*)&entropy, 8);
+   add_bytes(&entropy, 8);
    }
 
 /*************************************************
