@@ -12,6 +12,7 @@
 #include <map>
 #include <utility>
 #include <iostream>
+#include <iterator>
 
 #include <boost/tuple/tuple.hpp>
 
@@ -132,14 +133,14 @@ CMD(certs, "certs", "", CMD_REF(list), "ID",
 
       vector<string> lines;
       split_into_lines(washed, lines);
-      I(lines.size() > 0);
+      std::string value_first_line = lines.size() > 0 ? idx(lines, 0) : "";
 
       cout << string(guess_terminal_width(), '-') << '\n'
            << (i18n_format(str)
                % idx(certs, i).key()
                % stat
                % idx(certs, i).name()
-               % idx(lines, 0));
+               % value_first_line);
 
       for (size_t i = 1; i < lines.size(); ++i)
         cout << (i18n_format(extra_str) % idx(lines, i));
@@ -178,6 +179,8 @@ CMD(keys, "keys", "", CMD_REF(list), "[PATTERN]",
        i != pubs.end(); i++)
     pubkeys[*i] = true;
 
+  set<rsa_keypair_id> bad_keys;
+
   bool all_in_db = true;
   for (vector<rsa_keypair_id>::const_iterator i = privkeys.begin();
        i != privkeys.end(); i++)
@@ -187,6 +190,16 @@ CMD(keys, "keys", "", CMD_REF(list), "[PATTERN]",
           pubkeys[*i] = false;
           all_in_db = false;
         }
+      else if (app.db.database_specified())
+        {
+          // we've found a key that should have both a public and a private version
+          base64<rsa_pub_key> pub_key;
+          keypair priv_key;
+          app.db.get_key(*i, pub_key);
+          app.keys.get_key_pair(*i, priv_key);
+          if (!keys_match(*i, pub_key, *i, priv_key.pub))
+            bad_keys.insert(*i);
+        }
     }
 
   if (pubkeys.size() > 0)
@@ -235,6 +248,16 @@ CMD(keys, "keys", "", CMD_REF(list), "[PATTERN]",
       cout << '\n';
     }
 
+  if (!bad_keys.empty())
+    {
+      W(F("Some keys in the database have the same ID as, "
+          "but different hashes to, keys in your local key store!"));
+      for (set<rsa_keypair_id>::const_iterator i = bad_keys.begin(); i != bad_keys.end(); i++)
+        {
+          W(F("Mismatched Key: %s") % *i);
+        }
+    }
+
   if (pubkeys.size() == 0 &&
       privkeys.size() == 0)
     {
@@ -258,7 +281,7 @@ CMD(branches, "branches", "", CMD_REF(list), "[PATTERN]",
 
   globish exc(app.opts.exclude_patterns);
   set<branch_name> names;
-  app.get_project().get_branch_list(inc, names);
+  app.get_project().get_branch_list(inc, names, !app.opts.ignore_suspend_certs);
 
   for (set<branch_name>::const_iterator i = names.begin();
        i != names.end(); ++i)
