@@ -7,7 +7,7 @@
 // implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 // PURPOSE.
 
-#include <string>
+#include "base.hh"
 #include <map>
 #include <iostream>
 #include <unistd.h>
@@ -700,42 +700,43 @@ UNIT_TEST(key, arc4)
 
   SecureVector<Botan::byte> data(orig);
 
-  BOOST_CHECKPOINT("encrypting data");
+  UNIT_TEST_CHECKPOINT("encrypting data");
   do_arc4(phrase, data);
 
-  BOOST_CHECK(data != orig);
+  UNIT_TEST_CHECK(data != orig);
 
-  BOOST_CHECKPOINT("decrypting data");
+  UNIT_TEST_CHECKPOINT("decrypting data");
   do_arc4(phrase, data);
 
-  BOOST_CHECK(data == orig);
+  UNIT_TEST_CHECK(data == orig);
 
 }
 
 UNIT_TEST(key, signature_round_trip)
 {
   app_state app;
+  app.set_key_dir(system_path(get_current_working_dir()) / ".monotone_tmp" / "keys");
   app.lua.add_std_hooks();
   app.lua.add_test_hooks();
 
-  BOOST_CHECKPOINT("generating key pairs");
+  UNIT_TEST_CHECKPOINT("generating key pairs");
   keypair kp;
   utf8 passphrase("bob123@example.com");
   rsa_keypair_id key("bob123@example.com");
   generate_key_pair(kp, passphrase);
   app.keys.put_key_pair(key, kp);
 
-  BOOST_CHECKPOINT("signing plaintext");
+  UNIT_TEST_CHECKPOINT("signing plaintext");
   string plaintext("test string to sign");
   base64<rsa_sha1_signature> sig;
   make_signature(app, key, kp.priv, plaintext, sig);
 
-  BOOST_CHECKPOINT("checking signature");
-  BOOST_CHECK(check_signature(app, key, kp.pub, plaintext, sig));
+  UNIT_TEST_CHECKPOINT("checking signature");
+  UNIT_TEST_CHECK(check_signature(app, key, kp.pub, plaintext, sig));
 
   string broken_plaintext = plaintext + " ...with a lie";
-  BOOST_CHECKPOINT("checking non-signature");
-  BOOST_CHECK(!check_signature(app, key, kp.pub, broken_plaintext, sig));
+  UNIT_TEST_CHECKPOINT("checking non-signature");
+  UNIT_TEST_CHECK(!check_signature(app, key, kp.pub, broken_plaintext, sig));
   app.keys.delete_key(key);
 }
 
