@@ -4,10 +4,10 @@
 #include "mt_version.hh"
 
 extern sanity & global_sanity;
-mtn_sanity real_sanity;
+static mtn_sanity real_sanity;
 sanity & global_sanity = real_sanity;
 
-mtn_sanity::mtn_sanity() : relaxed(false)
+mtn_sanity::mtn_sanity()
 {}
 
 mtn_sanity::~mtn_sanity()
@@ -23,12 +23,6 @@ mtn_sanity::initialize(int argc, char ** argv, char const * lc_all)
   PERM_MM(full_version_string);
 }
 
-void
-mtn_sanity::set_relaxed(bool rel)
-{
-  relaxed = rel;
-}
-
 void
 mtn_sanity::inform_log(std::string const &msg)
 {
