@@ -34,7 +34,7 @@
 #include "keys.hh"
 #include "numeric_vocab.hh"
 #include "revision.hh"
-#include "mtn-sanity.hh"
+#include "sanity.hh"
 #include "transforms.hh"
 #include "simplestring_xform.hh"
 #include "ui.hh"
@@ -714,17 +714,16 @@ make_restricted_revision(parent_map const & old_rosters,
                          revision_t & rev)
 {
   edge_map edges;
-  cset dummy;
   for (parent_map::const_iterator i = old_rosters.begin();
        i != old_rosters.end();
        i++)
     {
       shared_ptr<cset> included(new cset());
-      make_restricted_csets(parent_roster(i), new_roster,
-                            *included, dummy, mask);
-      MM(*included);
-      MM(dummy);
-      check_restricted_cset(parent_roster(i), *included);
+      roster_t restricted_roster;
+
+      make_restricted_roster(parent_roster(i), new_roster, 
+                             restricted_roster, mask);
+      make_cset(parent_roster(i), restricted_roster, *included);
       safe_insert(edges, make_pair(parent_id(i), included));
     }
 
@@ -746,11 +745,12 @@ make_restricted_revision(parent_map const & old_rosters,
        i++)
     {
       shared_ptr<cset> included(new cset());
-      make_restricted_csets(parent_roster(i), new_roster,
-                            *included, excluded, mask);
-      MM(*included);
-      MM(excluded);
-      check_restricted_cset(parent_roster(i), *included);
+      roster_t restricted_roster;
+
+      make_restricted_roster(parent_roster(i), new_roster, 
+                             restricted_roster, mask);
+      make_cset(parent_roster(i), restricted_roster, *included);
+      make_cset(restricted_roster, new_roster, excluded);
       safe_insert(edges, make_pair(parent_id(i), included));
       if (!excluded.empty())
         no_excludes = false;
@@ -1646,7 +1646,6 @@ build_roster_style_revs_from_manifest_style_revs(app_state & app)
   app.db.ensure_open_for_format_changes();
   app.db.check_is_not_rosterified();
 
-  real_sanity.set_relaxed(true);
   anc_graph graph(true, app);
 
   P(F("converting existing revision graph to new roster-style revisions"));
@@ -1691,7 +1690,6 @@ build_roster_style_revs_from_manifest_style_revs(app_state & app)
       graph.add_node_for_oldstyle_revision(*i);
     }
 
-  real_sanity.set_relaxed(false);
   graph.rebuild_ancestry();
 }
 
