@@ -10,10 +10,11 @@
 // This file contains "diff"/"patch" code that operates directly on rosters
 // (with their associated markings).
 
+#include "base.hh"
 #include <set>
 #include <map>
 
-#include <boost/lexical_cast.hpp>
+#include "lexical_cast.hh"
 
 #include "safe_map.hh"
 #include "parallel_iter.hh"
