@@ -20,8 +20,8 @@
 #include <boost/scoped_array.hpp>
 
 #include "botan/botan.h"
-#include "botan/gzip.h"
 #include "botan/sha160.h"
+#include "gzip.hh"
 
 #include "cleanup.hh"
 #include "constants.hh"
